using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace SpaceTransformationExample
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {

        public const int SCREEN_WIDTH = 1080;
        public const int SCREEN_HEIGHT = 720;
        public const int NUM_STARS = 20;

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        WorldObject[] mPlanets;

        WorldObject[] mStars;
        WorldObject mCrosshair;

        Camera mCamera;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            graphics.PreferredBackBufferHeight = SCREEN_HEIGHT;
            graphics.PreferredBackBufferWidth = SCREEN_WIDTH;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            mPlanets = new WorldObject[4];
            mPlanets[0] = new WorldObject(Content.Load<Texture2D>("Sun"));
            mPlanets[0].PositionInParentSpace = new Vector2(300, 300);
            mPlanets[0].RotationInParentSpace = 1f;
            mPlanets[1] = new WorldObject(Content.Load<Texture2D>("Earth"), mPlanets[0]);
            mPlanets[1].PositionInParentSpace = new Vector2(150, 0);
            mPlanets[1].RotationInParentSpace = 1f;
            mPlanets[2] = new WorldObject(Content.Load<Texture2D>("Moon"), mPlanets[1]);
            mPlanets[2].RotationInParentSpace = 1f;
            mPlanets[2].PositionInParentSpace = new Vector2(50, 0);

            mPlanets[3] = new WorldObject(Content.Load<Texture2D>("Moon"), mPlanets[2]);
            mPlanets[3].PositionInParentSpace = new Vector2(40, 0);

            mCrosshair = new WorldObject(Content.Load<Texture2D>("crosshair"));
            mCrosshair.PositionInParentSpace = new Vector2(400, 400);

            mCamera = new Camera(spriteBatch);
            mStars = new WorldObject[NUM_STARS];
            Random r = new Random();
            for (int i = 0; i < NUM_STARS; i++)
            {
                if (r.NextDouble() > 0.7)
                    mStars[i] = new WorldObject(Content.Load<Texture2D>("StarLarge"));
                else
                    mStars[i] = new WorldObject(Content.Load<Texture2D>("StarSmall"));
                mStars[i].PositionInParentSpace = new Vector2((int)(r.NextDouble() * SCREEN_WIDTH),
                                                              (int)(r.NextDouble() * SCREEN_HEIGHT));
            }
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();
            KeyboardState kb = Keyboard.GetState();

            foreach (WorldObject w in mPlanets)
            {
                w.RotationInParentSpace += 0.02f;
            }

            Vector2 crosshairDelta = Vector2.Zero;
            if (kb.IsKeyDown(Keys.A))
            {
                crosshairDelta.X -= 3;
            }
            if (kb.IsKeyDown(Keys.D))
            {
                crosshairDelta.X += 3;
            }
            if (kb.IsKeyDown(Keys.W))
            {
                crosshairDelta.Y -= 3;
            }
            if (kb.IsKeyDown(Keys.S))
            {
                crosshairDelta.Y += 3;
            }

            if (kb.IsKeyDown(Keys.Left))
            {
                mCrosshair.RotationInParentSpace -= 0.01f;
            }
            if (kb.IsKeyDown(Keys.Right))
            {
                mCrosshair.RotationInParentSpace += 0.01f;
            }



            mCrosshair.PositionInParentSpace += crosshairDelta;


            for (int i = 0; i < 4; i++)
            {
                if (kb.IsKeyDown(Keys.D1 + i))
                {
                    mPlanets[i].RotationInWorldSpace = mCrosshair.RotationInParentSpace;
                    mPlanets[i].PositionInWorldSpace = mCrosshair.PositionInParentSpace;
                }
            }

            
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            mCamera.Begin();
            for (int i = 0; i < NUM_STARS; i++)
                mCamera.Draw(mStars[i]);

            foreach (WorldObject w in mPlanets)
            {
                mCamera.Draw(w);
            }
            mCamera.Draw(mCrosshair);
            mCamera.End();

            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }
    }
}
